@@ -18,7 +18,7 @@ use File::Path ();
 use Cwd ();
 use Config;
 
-our $VERSION = '0.350';
+our $VERSION = '0.351';
 
 use constant WIN32 => $^O eq 'MSWin32';
 
@@ -36,6 +36,14 @@ sub new {
         configure_timeout => 60,
         build_timeout => 3600,
         test_timeout => 1800,
+        with_requires => 1,
+        with_recommends => 0,
+        with_suggests => 0,
+        with_configure => 0,
+        with_build => 1,
+        with_test => 1,
+        with_runtime => 1,
+        with_develop => 0,
         %option
     }, $class;
 }
@@ -45,9 +53,13 @@ sub parse_options {
     local @ARGV = @_;
     $self->{notest} = 1;
     my (@mirror, @resolver);
+    my $with_option = sub {
+        my $n = shift;
+        ("with-$n", \$self->{"with_$n"}, "without-$n", sub { $self->{"with_$n"} = 0 });
+    };
     GetOptions
         "L|local-lib-contained=s" => \($self->{local_lib}),
-        "V|version" => sub { $self->cmd_version },
+        "V|version" => sub { $self->cmd_version; exit },
         "color!" => \($self->{color}),
         "g|global" => \($self->{global}),
         "h|help" => sub { $self->cmd_help },
@@ -64,13 +76,12 @@ sub parse_options {
         "dev" => \($self->{dev}),
         "man-pages" => \($self->{man_pages}),
         "home=s" => \($self->{home}),
-        "with-develop" => \($self->{with_develop}),
-        "with-recommends" => \($self->{with_recommends}),
-        "with-suggests" => \($self->{with_suggests}),
         "retry!" => \($self->{retry}),
         "configure-timeout=i" => \($self->{configure_timeout}),
         "build-timeout=i" => \($self->{build_timeout}),
         "test-timeout=i" => \($self->{test_timeout}),
+        (map $with_option->($_), qw(requires recommends suggests)),
+        (map $with_option->($_), qw(configure build test runtime develop)),
     or exit 1;
 
     $self->{local_lib} = $self->maybe_abs($self->{local_lib}) unless $self->{global};
@@ -161,9 +172,7 @@ sub cmd_help {
 }
 
 sub cmd_version {
-    my $class = ref $_[0] || $_[0];
-    printf "%s %s\n", $class, $class->VERSION;
-    exit 0;
+    print "cpm $VERSION ($0)\n";
 }
 
 sub cmd_exec {
@@ -187,6 +196,8 @@ sub cmd_install {
     File::Path::mkpath($self->{home}) unless -d $self->{home};
     my $logger = App::cpm::Logger::File->new("$self->{home}/build.log.@{[time]}");
     $logger->symlink_to("$self->{home}/build.log");
+    $logger->log("Running cpm $VERSION ($0) on perl $Config{version} built for $Config{archname} ($^X)");
+    $logger->log("Command line arguments are: @ARGV");
 
     my $master = App::cpm::Master->new(
         logger => $logger,
@@ -195,8 +206,8 @@ sub cmd_install {
     );
 
     # dryrun
-    $self->register_initial_job($master) or return 0;
-    $master->clear;
+    my ($packages, $dists) = $self->initial_job($master);
+    return 0 unless $packages;
 
     my $worker = App::cpm::Worker->new(
         verbose   => $self->{verbose},
@@ -223,8 +234,7 @@ sub cmd_install {
         ];
         my ($is_satisfied, @need_resolve) = $master->is_satisfied($requirements);
         last if $is_satisfied;
-        $master->add_job(type => "resolve", package => $_->{package}, version_range => $_->{version_range})
-            for @need_resolve;
+        $master->add_job(type => "resolve", %$_) for @need_resolve;
         $self->install($master, $worker, 1);
         %artifact = (%artifact, %{$master->{_artifacts}});
         $installed_distributions += $master->installed_distributions;
@@ -244,7 +254,8 @@ sub cmd_install {
         $master->clear;
     }
 
-    $self->register_initial_job($master) or return 0;
+    $master->add_job(type => "resolve", %$_) for @$packages;
+    $master->add_distribution($_) for @$dists;
     $self->install($master, $worker, $self->{workers});
 
     $installed_distributions += $master->installed_distributions;
@@ -324,64 +335,56 @@ sub cleanup {
     }
 }
 
-sub register_initial_job {
+sub initial_job {
     my ($self, $master) = @_;
 
-    my @package;
+    my (@package, @dist);
     for (@{$self->{argv}}) {
         my $arg = $_; # copy
+        my ($package, $dist);
         if (-d $arg || -f $arg || $arg =~ s{^file://}{}) {
             $arg = $self->maybe_abs($arg);
-            my $dist = App::cpm::Distribution->new(source => "local", uri => "file://$arg", provides => []);
-            $master->add_distribution($dist);
+            $dist = App::cpm::Distribution->new(source => "local", uri => "file://$arg", provides => []);
         } elsif ($arg =~ /(?:^git:|\.git(?:@.+)?$)/) {
             my %ref = $arg =~ s/(?<=\.git)@(.+)$// ? (ref => $1) : ();
-            my $dist = App::cpm::Distribution->new(source => "git", uri => $arg, provides => [], %ref);
-            $master->add_distribution($dist);
+            $dist = App::cpm::Distribution->new(source => "git", uri => $arg, provides => [], %ref);
         } elsif ($arg =~ m{^https?://}) {
-            my $dist = App::cpm::Distribution->new(source => "http", uri => $arg, provides => []);
-            $master->add_distribution($dist);
+            $dist = App::cpm::Distribution->new(source => "http", uri => $arg, provides => []);
         } else {
-            my ($package, $version_range, $dev);
+            my ($name, $version_range, $dev);
             # copy from Menlo
             # Plack@1.2 -> Plack~"==1.2"
             $arg =~ s/^([A-Za-z0-9_:]+)@([v\d\._]+)$/$1~== $2/;
             # support Plack~1.20, DBI~"> 1.0, <= 2.0"
             if ($arg =~ /\~[v\d\._,\!<>= ]+$/) {
-                ($package, $version_range) = split '~', $arg, 2;
+                ($name, $version_range) = split '~', $arg, 2;
             } else {
                 $arg =~ s/[~@]dev$// and $dev++;
-                $package = $arg;
+                $name = $arg;
             }
-            push @package, {package => $package, version_range => $version_range || 0, dev => $dev};
+            $package = {package => $name, version_range => $version_range || 0, dev => $dev};
         }
+        push @package, $package if $package;
+        push @dist, $dist if $dist;
     }
 
     if (!@{$self->{argv}}) {
-        my ($requirements, $dist) = $self->load_cpanfile($self->{cpanfile});
-        $master->add_distribution($_) for @$dist;
+        my ($requirements, $dists) = $self->load_cpanfile($self->{cpanfile});
+        push @dist, @$dists;
         my ($is_satisfied, @need_resolve) = $master->is_satisfied($requirements);
-        if (!@$dist and $is_satisfied) {
+        if (!@$dists and $is_satisfied) {
             warn "All requirements are satisfied.\n";
-            return 0;
+            return;
         } elsif (!defined $is_satisfied) {
             my ($req) = grep { $_->{package} eq "perl" } @$requirements;
             die sprintf "%s requires perl %s, but you have only %s\n",
                 $self->{cpanfile}, $req->{version_range}, $self->{target_perl} || $];
         } else {
-            @package = @need_resolve;
+            push @package, @need_resolve;
         }
     }
 
-    for my $p (@package) {
-        $master->add_job(
-            type => "resolve",
-            package => $p->{package},
-            version_range => $p->{version_range} || 0,
-            dev => $p->{dev},
-        );
-    }
-    return 1;
+    return (\@package, \@dist);
 }
 
 sub load_cpanfile {
@@ -389,12 +392,9 @@ sub load_cpanfile {
     require Module::CPANfile;
     my $cpanfile = Module::CPANfile->load($file);
     my $prereqs = $cpanfile->prereqs_with;
-    my $phases = [qw(build test runtime)];
-    push @$phases, 'develop' if $self->{with_develop};
-    my $types = [qw(requires)];
-    push @$types, 'recommends' if $self->{with_recommends};
-    push @$types, 'suggests' if $self->{with_suggests};
-    my $requirements = $prereqs->merged_requirements($phases, $types);
+    my @phase = grep $self->{"with_$_"}, qw(configure build test runtime develop);
+    my @type  = grep $self->{"with_$_"}, qw(requires recommends suggests);
+    my $requirements = $prereqs->merged_requirements(\@phase, \@type);
     my $hash = $requirements->as_string_hash;
 
     my (@package, @distribution);
