@@ -2,7 +2,7 @@ package App::cpm::Worker::Installer;
 use strict;
 use warnings;
 use utf8;
-our $VERSION = '0.350';
+our $VERSION = '0.351';
 
 use App::cpm::Logger::File;
 use App::cpm::Worker::Installer::Menlo;
@@ -75,6 +75,7 @@ sub new {
     $option{base}   ||= "$ENV{HOME}/.perl-cpm/work/" . time . ".$$";
     $option{cache}  ||= "$ENV{HOME}/.perl-cpm/cache";
     mkpath $_ for grep !-d, $option{base}, $option{cache};
+    $option{logger}->log("Work directory is $option{base}");
 
     my $menlo = App::cpm::Worker::Installer::Menlo->new(
         base => $option{base},
@@ -89,9 +90,12 @@ sub new {
         test_timeout => $option{test_timeout},
     );
     if (my $local_lib = delete $option{local_lib}) {
+        $local_lib = $menlo->maybe_abs($local_lib);
         $menlo->{self_contained} = 1;
-        $menlo->setup_local_lib($menlo->maybe_abs($local_lib));
+        $menlo->log("Setup local::lib $local_lib");
+        $menlo->setup_local_lib($local_lib);
     }
+    $menlo->log("--", `$^X -V`, "--");
     bless { %option, menlo => $menlo }, $class;
 }
 
